/*********************************************************************
 *              Copyright (c) 2021 PikeTec GmbH
 *               CONFIDENTIAL AND PROPRIETARY.
 *                   ALL RIGHTS RESERVED
 *              - PROPRIETARY OF PikeTec GmbH -
 **********************************************************************/
/* clang-format off */

/*
 * declares error codes and error handling auxillary functions used by the TPT VM and derived APIs.
 * 
 * PikeTec tries to keep this interface as compatible as possible between TPT 
 * versions without causing users too much pain when migrating from one version to the next. 
 * Nevertheless, compatibility cannot always be 100% guaranteed. Therefore, all users of 
 * this API are advised to check compatibility and adjust if necessary before switching to 
 * a new version.
 */

#ifndef __TPT_ERRORS_H_
#define __TPT_ERRORS_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ------------------------- */



#include "TPT_Types.h"


/*
 * Optional function that is called implicitly when an error is triggered using `tpt_raiseError()` so that the error can 
 * be reported and handled. The error handler is allowed to *immediately interrupt* the control flow (if desired).
 * In other words, a caller of `tpt_raiseError()` cannot rely on `tpt_raiseError()` to return regularily.
 * 
 * (Note: Implementators of an error handler fimctopm can use features such as exit(), longjmp(), raise(), or 
 * exception handling to interrupt the control flow, to drop the call stack and to jump directly to the appropriate
 * error recovery code.)
 * 
 * If no error handler is installed using `tpt_setErrorHandler()` then error messages are not reported immediately,
 * but merely stored in `tpt_getErrorMessage()` and `tpt_getErrorCode()`. So, using an error handler is an optional
 * feature.
 * 
 * @see tpt_setErrorHandler()
 * @see tpt_raiseError()
 */
typedef void (*tpt_errorhandler)();



/*
 * Optional function that is called implicitly when an error is triggered using `tpt_raiseError()` to provide the 
 * current "call stack context". This "call stack context" usually contains, for example, stack information, module
 * name, time, program counter, thread ID etc (as human readable text). This textual description (if non-empty) will be 
 * automatically appended to the error message to constitute a meaningful error message.
 * 
 * If called, the error context handler should fill the buffer `buf` (of size `bufsize`) with a textual description
 * of the "call stack context" where this error occured. The length `bufsize` must not be exceeded including the 
 * terminating \0 character (to avoid segfaults). `buf` is guaranteed to contain at least 4096 characters 
 * (`bufsize >= 4096) which should be sufficient in most cases.
 * 
 * If no error context handler is installed using `tpt_setErrorContextHandler()` then error messages simply do not 
 * contain context information. So, using an error context handler is an optional feature.
 * 
 * @param buf
 *    the character buffer to write the context information into. This buffer contains at least 4096 characters. The 
 *    precise size is stored in `bufsize`
 * @param bufsize
 *    the actual size of the buffer `buf` (guaranteed is: bufsize >= 4096)
 * 
 * @see tpt_setErrorContextHandler()
 * @see tpt_raiseError()
 */
typedef void (*tpt_errorcontexthandler)(char* buf, tpt_uint32 bufsize);



/*
 * delivers the current error code. If there is no error, the value `tpt_err_noerror` will be returned.
 * In case of errors, one of the error codes with prefix `tpt_err_xxxx` will be returned 
 * (these constants are defined in this header file as well). If there is an error code != `tpt_err_noerror` 
 * then a detailed human readable error message can be obtained using `tpt_getErrorMessage()`.
 */
TPT_DECLARE(tpt_uint16) tpt_getErrorCode();



/*
 * `tpt_true` if an error occured recently. This is a shortcut for `tpt_getErrorCode() != tpt_err_noerror`
 */
TPT_DECLARE(tpt_bool) tpt_hasError();



/*
 * delivers a human readable error message, if `tpt_hasError() == tpt_true`. Note, that this function
 * returns a pointer to a buffer that might be disposed or changed later by TPT. If the caller needs this message
 * later again, it should be copied.
 */
TPT_DECLARE(const char*) tpt_getErrorMessage();



/*
 * installs or changes the error handler. This error handler will be called as soon as an error is raised by calling
 * `tpt_raiseError()`. The TPT VM uses `tpt_raiseError()` as its central error reporting mechanism. Error handlers 
 * usually log error messages and (optionally) immediately interrupt the control flow. Therefore, a caller of 
 * `tpt_raiseError()` cannot rely on `tpt_raiseError()` to return regularily.
 * 
 * 
 * (Note: Implementators of an error handler fimctopm can use features such as exit(), longjmp(), raise(), or 
 * exception handling to interrupt the control flow, to drop the call stack and to jump directly to the appropriate
 * error recovery code.)
 * 
 * If no error handler is installed using `tpt_setErrorHandler()` (default is TPT_NULL) then error messages are not 
 * reported immediately, but merely stored in `tpt_getErrorMessage()` and `tpt_getErrorCode()`. So, using an error 
 * handler is an optional feature.
 * 
 * The error handler can immediately query the code and messages of the error occured by means of `tpt_getErrorCode()` 
 * and `tpt_getErrorMessage()`.
 *
 * @param errorHandler 
 *    the callback function to be notified in case of an error. TPT_NULL will turn off this notification
 *    (reset to "no direct error notification/handling").
 * 
 * @return 
 *    returns the error handler that has been installed before. If the new `errorHandler` should be 
 *    "removed/uninstalled" for some reasons later, it is important for the caller to re-set the handler to 
 *    this previous handler.
 * 
 * @see tpt_raiseError()
 */
TPT_DECLARE(tpt_errorhandler) tpt_setErrorHandler(tpt_errorhandler errorHandler);



/*
 * installs or changes the error context handler. Each time an error will be raised using `tpt_raiseError()` this 
 * error context handler (if installed) will be requested to deliver some additional context information for this
 * error (e.g. stack information, time, program counter, thread ID etc.) that will be appended to the error
 * message (see `tpt_getErrorMessage()`.
 *
 * @param errorContextHandler
 *    the callback function to be requested for context informaton in case of an error. null will turn of this 
 *    context retrieval.
 * 
 * @return 
 *    the error context handler that has been installed before. It is good practice of a caller to keep this
 *    reference to the previous handler and to re-set it *after* the current context has been left.
 * 
 * @see tpt_raiseError()
 */
TPT_DECLARE(tpt_errorcontexthandler) tpt_setErrorContextHandler(tpt_errorcontexthandler errorContextHandler);



/*
 * raises an error with the given error code and error message. Usually this function is not for user
 * purpose. Mainly the TPT VM uses this function to signal errors. However, it can also be used, for
 * example, in custom functions to signal semantic errors to the VM control flow.
 * 
 * tpt_raiseError() does the following: 
 * (1) error code and error message are stored to be accessible using `tpt_getErrorCode()`
 *     and `tpt_getErrorMessage()`.
 * (2) if an error context handler has been installed using `tpt_setErrorContextHandler()`, the error context
 *     is retrieved from this handler and appended to the error message.
 * (3) if an error handler has been installed using `tpt_setErrorHandler()`, this error handler is triggered
 * 
 * Note that, by design, the call of an error handler (step 3 above) is allowed to *interrupt* the control flow. 
 * Therefore, a caller of `tpt_raiseError()` cannot rely on `tpt_raiseError()` to return regularily.
 *
 * @param errorcode 
 *     one of the constants <code>tpt_error_xxx</code> declared in this header file <code>TPT_Errors.h</code>
 * @param fmt
 *     the error message can be formatted using the same formatting options as for standard printf()
 * 
 * @see typedef of tpt_errorhandler
 */
TPT_DECLARE(void) tpt_raiseError(tpt_uint16 errorcode, const char* fmt, ...);



/*
 * reset of error code and error message. Users of this API usually do not have to reset the error status.
 * This is done automatically if one of the TPT VM functions is called next time. However, the effect of this
 * function is quite simple: errorcode = tpt_err_noerror and errormsg = NULL.
 */
TPT_DECLARE(void) tpt_resetError();



/*
* general purpose print-function.
* Output of a logging or error message to console (stdout) or some other appropriate output device that allows human
* readable feedback to the user. The implementation is system/platform dependent. In some case is can also be empty 
* (omitting the message) if the system does not support any output logging stream.
* 
* @param fmt 
*    format and vararg list supports the same features as standard printf() function.
* @return
     the length of the string output to the console
*/
TPT_DECLARE(int) tpt_printf(char* fmt, ...);




/* ------------------- TPT ERROR CODES --------------------- */
#define tpt_err_noerror                                 ((tpt_uint16)0)         /* no error */
#define tpt_err_magic_key                               ((tpt_uint16)10)        /* wrong data type of the TPT VM program file */
#define tpt_err_tvmformat                               ((tpt_uint16)11)        /* tvm format error (wrong version) */
#define tpt_err_cycle_no_following                      ((tpt_uint16)12)        /* runtime error if a state terminated, but none of the subsequent transitions fires (blocking) */
#define tpt_err_cycle_two_transitions_same_time         ((tpt_uint16)13)        /* runtime error if two concurrent transitions fire at the same time (race condition) */
#define tpt_err_action_code_terminates                  ((tpt_uint16)14)        /* action of a transition terminates unexpectedly */
#define tpt_err_heap_to_small                           ((tpt_uint16)15)        /* static heap size is smaller than needed for the current program */
#define tpt_err_future_value                            ((tpt_uint16)16)        /* access to a signal value in the future (signal(t + n) with n &gt; 0) */
#define tpt_err_illegal_datatype                        ((tpt_uint16)18)        /* illegal parameter in current OP-code */
#define tpt_err_io                                      ((tpt_uint16)19)        /* error while accessing the file system */
#define tpt_err_cycle_accessing_unwritten_channel       ((tpt_uint16)100)       /* access to an input signal that could not be loaded to TPT (access problem) */
#define tpt_err_channel_value_not_longer_available      ((tpt_uint16)101)       /* access to a signal that is no longer in history queue (history size too small) */
#define tpt_err_jump_table                              ((tpt_uint16)102)       /* index missing in jump table ("array index out of bounds") */
#define tpt_err_stdfunction_unknown                     ((tpt_uint16)103)       /* standard function called with illegal function code */
#define tpt_err_customfunction_unknown                  ((tpt_uint16)103)       /* custom function called (server stub or client) without an installed handler function */
#define tpt_err_cast_error                              ((tpt_uint16)104)       /* value out of range while cast is performed */
#define tpt_err_signed_cast                             ((tpt_uint16)105)       /* error in unsigned / signed conversion, value out of range */
#define tpt_err_division_by_zero                        ((tpt_uint16)106)       /* division or modulo by zero */
#define tpt_err_stack_overflow                          ((tpt_uint16)108)       /* stack overflow */
#define tpt_err_stack_underflow                         ((tpt_uint16)109)       /* stack underflow */
#define tpt_err_callstack_overflow                      tpt_err_stack_overflow  /* legacy: mapped to tpt_err_stack_overflow */
#define tpt_err_parastack_overflow                      tpt_err_stack_overflow  /* legacy: mapped to tpt_err_stack_overflow */
#define tpt_err_parastack_underflow                     tpt_err_stack_underflow /* legacy: mapped to tpt_err_stack_underflow */
#define tpt_err_table_index_not_found                   ((tpt_uint16)110)       /* legacy: not in use any longer */
#define tpt_err_array_index_out_bounds                  ((tpt_uint16)111)       /* array out of bounds */
#define tpt_err_index_not_monoton                       ((tpt_uint16)112)       /* monotony violation in CURVE/MAP interpolation */
#define tpt_err_parastack_mismatch                      ((tpt_uint16)113)       /* parameter stack size differs before/after function call */
#define tpt_err_callstack_mismatch                      ((tpt_uint16)114)       /* call stack size differs before/after function call */
#define tpt_err_density_frozen                          ((tpt_uint16)115)       /* tpt_setDensity() called after TPT VM has been started already */
#define tpt_err_historysize_frozen                      ((tpt_uint16)116)       /* legacy: not in use any longer */
#define tpt_err_opcode_unknown                          ((tpt_uint16)117)       /* unknown OP code in TVM program */
#define tpt_err_no_msgbox_handler                       ((tpt_uint16)118)       /* msg-box command has been triggered but no MSGBOX-Handler has been installed before using `tpt_installMessageBoxCmd()` */
#define tpt_err_bind                                    ((tpt_uint16)119)       /* `tpt_vmapi_bindSignal()` failed because the signal does not exist or has a different data type */
#define tpt_err_null_pointer                            ((tpt_uint16)120)       /* a function receives a NULL pointer or handle for one of the API functions that does not allowed NULL values */
#define tpt_err_shared_memory                           ((tpt_uint16)121)       /* shared memory could not be installed correctly */
#define tpt_err_string_too_long                         ((tpt_uint16)122)       /* a string does not fit to the target array (too many characters in the string) */
#define tpt_err_general                                 ((tpt_uint16)123)       /* general purpose error */
#define tpt_err_arithmetic                              ((tpt_uint16)124)       /* general purpose arithmetic error (error in computation of an arithmetic expression) */
#define tpt_err_error_in_custom_function                ((tpt_uint16)10000)     /* an error occured inside an external custom function */



#ifdef __cplusplus
}
#endif

#endif /* __TPT_ERRORS_H_ */
