/*********************************************************************
|  ----------------------------------------------------------------
|              Copyright (c) 2020 PikeTec GmbH
|  CONFIDENTIAL AND PROPRIETARY.ALL RIGHTS RESERVED -PROPRIETARY OF
|                       PikeTec GmbH
|********************************************************************/


static DTypeId tpt_ss_int64_type = 0;
static tpt_bool tpt_ss_int64_type_initialized = tpt_false;

static DTypeId tptInitInt64Type(SimStruct* S) {
  if(!tpt_ss_int64_type_initialized) {
    tpt_ss_int64_type = ssRegisterDataTypeFxpBinaryPoint(S, tpt_true, 64, 0, tpt_false);
    tpt_ss_int64_type_initialized = tpt_true;
  }
  return tpt_ss_int64_type;
}

static tpt_bool tptIsInt64TypeInitalized() {
  return tpt_ss_int64_type_initialized;
}
static DTypeId tptGetInt64Type() {
  return tpt_ss_int64_type;
}

static void tptReportError(void *simStructPtr, char *msg){
	ssSetErrorStatus((SimStruct*)simStructPtr, msg);
}

/**
* Recursively analyzes a bus type, count the maximum size for the dims array, the maximum bus-type depth and number of leaf-signals
*/
static tpt_bool tptCountNumOfBusSignals(SimStruct* S, DTypeId typeId,
  tpt_uint8 *numMaxDims, tpt_uint16 *numMaxBusDepth, tpt_uint16 *numOfLeafSignals){
    
  int_T numBusElements;
  int_T iElem;
  tpt_uint8 maxElemDims = 0;
  tpt_uint16 maxElemBusDepth = 0;
  
  if (ssIsDataTypeABus(S,typeId) != 1){
    return tpt_false;
  }
  numBusElements = ssGetNumBusElements(S, typeId);
  
  for(iElem=0 ; iElem < numBusElements; iElem++){
    DTypeId elemType; 
    int_T elemNDims;
    const int_T *elemDims;
    tpt_uint8 numElemDims; /* number of dimensions and struct-levels from this elemen to the leaf element */
    tpt_uint16 elemBusDepth;
    elemType = ssGetBusElementDataType(S, typeId, iElem);
    elemNDims = ssGetBusElementNumDimensions(S, typeId, iElem);
    elemDims = ssGetBusElementDimensions(S, typeId, iElem);
    elemBusDepth = 1;
    
    if (elemNDims==1){
      if(elemDims[0] > 1) {
        numElemDims = 1; /* array */
      } else {
        numElemDims = 0; /* scalar */
      }
    } else if( elemNDims==2) {
      numElemDims = 2; /* matrix */
    } else{
      return tpt_false;
    }
    
    if (ssIsDataTypeABus(S,elemType) == 1){
      if(!tptCountNumOfBusSignals(S, elemType, &numElemDims, &elemBusDepth, numOfLeafSignals))
        return tpt_false;
    } else {
      *numOfLeafSignals = *numOfLeafSignals+1;
    }
    if(numElemDims > maxElemDims) {
      /* calc maximum number of dims over all elements */
      maxElemDims = numElemDims;
    }
    if(elemBusDepth > maxElemBusDepth) {
      /* calc maximum bus depth over all elements */
      maxElemBusDepth = elemBusDepth;
    }
  }
  *numMaxBusDepth = *numMaxBusDepth+maxElemBusDepth;
  *numMaxDims = *numMaxDims+maxElemDims+1;/* add 1 for struct level */
  return tpt_true;
}

/**
* Get the tpt base type from a given primitive MATLAB type id
*/
static tpt_basetype getTptBaseType(SimStruct* S, DTypeId typeId, tpt_bool *isFixPtType){
  typeId = ssGetDataTypeIdAliasedThruTo(S, typeId);
  switch(typeId){
    case SS_DOUBLE:
      return tpt_type_double;
    case SS_SINGLE:
      return tpt_type_float;
    case SS_INT8:
      return tpt_type_int8;
    case SS_INT16:
      return tpt_type_int16;
    case SS_INT32:
      return tpt_type_int32;
    case SS_UINT8:
      return tpt_type_uint8;
    case SS_UINT16:
      return tpt_type_uint16;
    case SS_UINT32:
      return tpt_type_uint32;
    case SS_BOOLEAN:
      return tpt_type_bool;
    default: {
      if(ssGetDataTypeIsFxpFltApiCompat(S, typeId)) {
        if(ssGetDataTypeIsFloatingPoint(S, typeId)){
          int_T dtSize = ssGetDataTypeSize(S, typeId);
          if(dtSize == 4) {
            return tpt_type_float; /* single type in ML */
          } else if(dtSize == 8) {
            return tpt_type_double; /* double type in ML */
          }
        } else if(ssGetDataTypeIsFixedPoint(S, typeId)) {
          if(ssGetDataTypeIsScalingTrivial(S, typeId)) {
            tpt_bool isSigned = ssGetDataTypeFxpIsSigned(S, typeId);
            switch(ssGetDataTypeSize(S, typeId)) {
              case 1:
                return isSigned ? tpt_type_int8 : tpt_type_uint8;
              case 2:
                return isSigned ? tpt_type_int16 : tpt_type_uint16;
              case 4:
                return isSigned ? tpt_type_int32 : tpt_type_uint32;
              case 8: {
                if(isSigned) {
                  tptInitInt64Type(S);
                  return tpt_type_int64;
                } else {
                  tptInitInt64Type(S);
                  /* uint64 type not supported natively use int64 in TPT and handle as normal fixedpoint type */
                  *isFixPtType = tpt_true;
                  return tpt_type_int64;
                }
              }
              default: {
                /* non-standard byte size, handle as normal fixedpoint type */
                *isFixPtType = tpt_true;
                return tpt_type_double;
              }
            }
          }
          
          *isFixPtType = tpt_true;
          return tpt_type_double; /* scaled type, use double in TPT */
        }
      }
      return getEnumBaseTypeByName(ssGetDataTypeName(S, typeId));
    }
  }
}

static DTypeId getMlBuiltInType(tpt_basetype tptVMType){
  switch(tptVMType){
    case tpt_type_double:
      return SS_DOUBLE;
    case tpt_type_float:
      return SS_SINGLE;
    case tpt_type_int8:
      return SS_INT8;
    case tpt_type_int16:
      return SS_INT16;
    case tpt_type_int32:
      return SS_INT32;
    case tpt_type_int64:
      return tpt_ss_int64_type;/* if we have an int64 type this was initialized before */
    case tpt_type_uint8:
      return SS_UINT8;
    case tpt_type_uint16:
      return SS_UINT16;
    case tpt_type_uint32:
      return SS_UINT32;
    case tpt_type_bool:
      return SS_BOOLEAN;
    default: 
      return -1; /* should not happen, since input comes from TPT VM */
  }
}

/**
* Recursively analyzes a bus type and fills the <outTptPortData> array with all needed information on the leaf signals
*/
static void tptGetBusElementsDataRec(SimStruct* S, 
  tpt_uint32* leafElemIdx, DTypeId typeId, char* signalName,  int_T addressOffset, 
  tpt_tsize* tptDimsRec, int idxDimsRec, DTypeId* typeIdsRec, int busDepth,
  tpt_sl_port_data *mlPortData, tpt_port_data * outTptPortData ){
  
  int_T numBusElements;
  int iElem;
  
  if (ssIsDataTypeABus(S,typeId) != 1){
    return;/* should not happen */
  }
  
  numBusElements = ssGetNumBusElements(S, typeId);
  for(iElem=0 ; iElem < numBusElements; iElem++){
    DTypeId elemType; 
    int_T elemNDims;
    int_T elemDimsIdxRec;
    int_T elemOffset;
    const int_T *elemDims;
    const char *elemName;
    char *elemFullName;
    elemType = ssGetBusElementDataType(S, typeId, iElem);
    elemNDims = ssGetBusElementNumDimensions(S, typeId, iElem);
    elemDims = ssGetBusElementDimensions(S, typeId, iElem);
    elemName = ssGetBusElementName(S, typeId, iElem);
    elemOffset = ssGetBusElementOffset(S, typeId, iElem);
    elemFullName = tpt_sfun_malloc(sizeof(char)*((tpt_uint32)strlen(signalName)+(tpt_uint32)strlen(elemName)+2)); /* <signalName>.<elemenName> */
    sprintf(elemFullName, "%s.%s", signalName, elemName);
    
    elemDimsIdxRec = idxDimsRec;
    typeIdsRec[busDepth] = elemType;
    
    if (elemNDims==1){
      if(elemDims[0] > 1) {
        /* array */
        tptDimsRec[elemDimsIdxRec] = elemDims[0];
        elemDimsIdxRec = elemDimsIdxRec+1;
      } else {
        /* scalar */
      }
    } else if( elemNDims==2) { 
      /* matrix with dimension [n m] */
      if( elemDims[0] > 1 || elemDims[1] > 1 ) {
        /* only if n > 1 or m > 1 we treat it as a matrix in TPT */
        tptDimsRec[elemDimsIdxRec] = elemDims[0];
        elemDimsIdxRec = elemDimsIdxRec+1;
        tptDimsRec[elemDimsIdxRec] = elemDims[1];
        elemDimsIdxRec = elemDimsIdxRec+1;
      } else {
      /* with n==1 and m==1 we treat it as scalar */
      }
    } else{
      /* multidim array, unsupported, should have been checked before */
    }
    
    if (ssIsDataTypeABus(S,elemType) == 1){
      /* mark struct level with uint16 max-value */
      tptDimsRec[elemDimsIdxRec] = DIMS_STRUCT_ELEM; /* uint16 max */
      tptGetBusElementsDataRec(S,  
        leafElemIdx, elemType, elemFullName, addressOffset+elemOffset, tptDimsRec, elemDimsIdxRec+1, typeIdsRec, busDepth+1,
        mlPortData, outTptPortData);
      /* memory for signal name not used on this level any more, free it here */
      tpt_sfun_free(elemFullName); 
    } else {
      int iDim, iType;
      tpt_uint32* slTypeIds;/* do not use DTypeId since this goes into TPT internal data structure */
      tpt_uint32* slTypeSizes;
      tpt_bool hasArrayData;
      
      hasArrayData = tpt_false;
      for(iDim=0; iDim < elemDimsIdxRec; iDim++) {
        if(tptDimsRec[iDim] < DIMS_STRUCT_ELEM) {
          hasArrayData = tpt_true; 
          break;
        }
      }
      if(hasArrayData) {
        /* only use dims array if we actually have an array at lease once somewhere */
        tpt_tsize* tptDims = tpt_sfun_malloc(sizeof(tpt_tsize)*(elemDimsIdxRec)); /* copy the current data from the typeIdsRec */
        memcpy(tptDims, tptDimsRec, sizeof(tpt_tsize)*(elemDimsIdxRec));
        outTptPortData->signals[*leafElemIdx].nDims = elemDimsIdxRec;
        outTptPortData->signals[*leafElemIdx].dims = tptDims;
      } else {
        outTptPortData->signals[*leafElemIdx].nDims = 0;
        outTptPortData->signals[*leafElemIdx].dims = TPT_NULL;
      }
      
      outTptPortData->signals[*leafElemIdx].name = elemFullName; 
      
      /* store the datatype ids and sizes */
      outTptPortData->signals[*leafElemIdx].nSlTypes = busDepth+1;
      slTypeIds = tpt_sfun_malloc(sizeof(tpt_uint32)*(busDepth+1)); 
      slTypeSizes = tpt_sfun_malloc(sizeof(tpt_uint32)*(busDepth+1)); 
      
      for(iType = 0; iType < busDepth+1; iType++) {
        slTypeIds[iType] = typeIdsRec[iType];
        slTypeSizes[iType] = ssGetDataTypeSize(S, typeIdsRec[iType]);
      }
      outTptPortData->signals[*leafElemIdx].slTypeIds = slTypeIds;
      outTptPortData->signals[*leafElemIdx].slTypeSizes = slTypeSizes;
      
      outTptPortData->signals[*leafElemIdx].isScaledFixedPointType = tpt_false;
      outTptPortData->signals[*leafElemIdx].dataType = getTptBaseType(S, elemType, 
        &outTptPortData->signals[*leafElemIdx].isScaledFixedPointType);
      
      outTptPortData->signals[*leafElemIdx].mlBusElementOffset = addressOffset+elemOffset;
      
      /* might be set by inner S-function later on, if required */
      outTptPortData->signals[*leafElemIdx].nMlElementOffsets = 0;
      outTptPortData->signals[*leafElemIdx].mlElementOffsets = TPT_NULL;
      
      mlPortData->leafElemMlTypes[*leafElemIdx] = elemType;

      *leafElemIdx = *leafElemIdx+1;
    }
  }
  return;
}

static void tptGetBusElementsData(SimStruct* S, 
  DTypeId typeId, char* signalName, tpt_tsize* tptDimsRec, int idxDimsRec, DTypeId* typeIdsRec, int busDepth,
  tpt_sl_port_data *mlPortData, tpt_port_data * outTptPortData){
  tpt_uint32 leafElemIdx = 0;

  tptGetBusElementsDataRec(S, &leafElemIdx,
    typeId, signalName, 0, tptDimsRec, idxDimsRec, typeIdsRec, busDepth,
    mlPortData, outTptPortData);
}

static void cleanupPortData(tpt_port_data *portData, tpt_sl_port_data *mlPortData){
  int iSig;
  if(portData->dataInitialized){
    for(iSig = 0; iSig < portData->nPortSignals; iSig++){
      tpt_sfun_free( portData->signals[iSig].name);
      tpt_sfun_free( portData->signals[iSig].slTypeSizes);
      if (portData->signals[iSig].nDims > 0) {
        tpt_sfun_free( portData->signals[iSig].dims);
      }
      if(portData->signals[iSig].nMlElementOffsets>0) {
        /* if used the number of eleemnts is set to a value > 0 */
        tpt_sfun_free(portData->signals[iSig].mlElementOffsets);
        portData->signals[iSig].mlElementOffsets = TPT_NULL;
        portData->signals[iSig].nMlElementOffsets = 0;
      }
    }
    tpt_sfun_free(portData->signals);
    portData->signals = TPT_NULL;
    portData->nPortSignals = 0;
    
    tpt_sfun_free(mlPortData->leafElemMlTypes);
    mlPortData->leafElemMlTypes = TPT_NULL;
    mlPortData->nLeafElems = 0;
    mlPortData->portType = 0;
    
    portData->dataInitialized = tpt_false;
    tpt_ss_int64_type_initialized = tpt_false;
  }
}

static void tptCleanupPortInterfaceData(tpt_in_out *ifData, tpt_sl_port_data *slInportData, tpt_sl_port_data *slOutportData, 
  tpt_bool **fcnCallStates, tpt_bool **fcnCallStatesOld, tpt_tsize *nFcnCallStates){
  int iPort;
  
  /* clean up port data details */
  /* do not clean up these arrays themselves as this is constant memory */
  for(iPort = 0; iPort < ifData->tpt_ins_size; iPort++){
    cleanupPortData(&ifData->tpt_ins_data[iPort], &slInportData[iPort]);
  }
  for(iPort = 0; iPort < ifData->tpt_outs_size; iPort++){
    cleanupPortData(&ifData->tpt_outs_data[iPort], &slOutportData[iPort]);
  }
 
  if(*nFcnCallStates>0) {
    tpt_sfun_free(*fcnCallStates);
    tpt_sfun_free(*fcnCallStatesOld);
    *nFcnCallStates = 0;
  }
}


/**
* Initializes all internal port data for one given port based on the given data type and dimensions
*/
static tpt_bool tptInitTptPortData(SimStruct* S, char* signalName, tpt_sl_port_data *mlPortData, int dim1, int dim2, tpt_bool isNonVirtualBus, tpt_port_data * outTptPortData){
  tpt_bool success = tpt_true;
  tpt_uint8 nPortDims; /* number of dims directly defined on this port */
  tpt_tsize *portDimsTmp;

  tpt_uint8 nDimsMax; /* maximum length of dims array for elements of this signal */
  tpt_uint16 numOfLeafSignals;
  tpt_uint16 maxBusDepth;

  outTptPortData->isNonVirtualBus = isNonVirtualBus;
  outTptPortData->writeInitialValue = tpt_false;

  if(dim2 > 0) {
    nPortDims = 2; /* matrix at port */
    portDimsTmp = tpt_sfun_malloc(nPortDims * sizeof(tpt_tsize));
    portDimsTmp[0] = dim1;
    portDimsTmp[1] = dim2;
  } else if(dim1>1) {
    nPortDims = 1; /* array at port level */
    portDimsTmp = tpt_sfun_malloc(nPortDims * sizeof(tpt_tsize));
    portDimsTmp[0] = dim1;
  } else {
    nPortDims=0;
    portDimsTmp = TPT_NULL;
  }
  nDimsMax = nPortDims;

  maxBusDepth = 1;

  if (ssIsDataTypeABus(S,mlPortData->portType) == 1){
    numOfLeafSignals = 0;
    success = tptCountNumOfBusSignals(S, mlPortData->portType, &nDimsMax, &maxBusDepth, &numOfLeafSignals);
  }else{
    numOfLeafSignals=1;
  }
  
  if(success) {
    /* init memory for port data arrays */
    outTptPortData->nPortSignals = numOfLeafSignals;
    outTptPortData->signals = tpt_sfun_malloc(numOfLeafSignals * sizeof(tpt_port_signal));
    mlPortData->nLeafElems = numOfLeafSignals;
    mlPortData->leafElemMlTypes = tpt_sfun_malloc(numOfLeafSignals * sizeof(DTypeId));

    if (ssIsDataTypeABus(S,mlPortData->portType) == 1){
      DTypeId* typeIdsRecBuffer;
      tpt_tsize* dimsRecBuffer;

      typeIdsRecBuffer =  tpt_sfun_malloc(maxBusDepth * sizeof(DTypeId));
      typeIdsRecBuffer[0] = mlPortData->portType;

      /* buffer for dims array, preconfigure with dimensions for port */
      dimsRecBuffer = tpt_sfun_malloc(nDimsMax * sizeof(tpt_tsize));
      if(nPortDims > 0) {
        /* copy dimensions on port to buffer */
        memcpy(dimsRecBuffer, portDimsTmp, nPortDims * sizeof(tpt_tsize));
      }
      dimsRecBuffer[nPortDims] = DIMS_STRUCT_ELEM;/* marker for first struct level */

      tptGetBusElementsData(S, mlPortData->portType, signalName, dimsRecBuffer, nPortDims+1, typeIdsRecBuffer, 1, mlPortData, outTptPortData);
      tpt_sfun_free(typeIdsRecBuffer);
      tpt_sfun_free(dimsRecBuffer);
    }else {
      /* one element only, use 0 as index for all */
      /* allways allocate new memory and copy the name, since we allways free the memory for the name later on */
      char* sigNameCpy = tpt_sfun_malloc(sizeof(char)*((tpt_uint32)strlen(signalName)+1));
      strcpy(sigNameCpy, signalName);
      outTptPortData->signals[0].name = sigNameCpy;
      outTptPortData->signals[0].isScaledFixedPointType = tpt_false;
      outTptPortData->signals[0].dataType = getTptBaseType(S, mlPortData->portType,
        &outTptPortData->signals[0].isScaledFixedPointType);
      outTptPortData->signals[0].nSlTypes = 1;
      outTptPortData->signals[0].slTypeIds = tpt_sfun_malloc(sizeof(tpt_uint32));/* type ids stored as uint32 in tpt-internal data */
      outTptPortData->signals[0].slTypeIds[0] = mlPortData->portType;
      outTptPortData->signals[0].slTypeSizes = tpt_sfun_malloc(sizeof(tpt_uint32));
      outTptPortData->signals[0].slTypeSizes[0] = ssGetDataTypeSize(S, mlPortData->portType);
      
      outTptPortData->signals[0].nDims = nPortDims;
      if(nPortDims>0) {
        outTptPortData->signals[0].dims = tpt_sfun_malloc(nPortDims * sizeof(tpt_tsize));
        memcpy(outTptPortData->signals[0].dims, portDimsTmp, nPortDims * sizeof(tpt_tsize));
      } else {
        outTptPortData->signals[0].dims = TPT_NULL;
      }
      /* might be set by inner S-function later on, if required */
      outTptPortData->signals[0].nMlElementOffsets = 0;
      outTptPortData->signals[0].mlElementOffsets = TPT_NULL;
      
      mlPortData->leafElemMlTypes[0] = mlPortData->portType;
    }
    outTptPortData->dataInitialized = tpt_true;
  }
  if(portDimsTmp!= TPT_NULL) {
    tpt_sfun_free(portDimsTmp);
  }
  return success;
}

/**
* Initialize an input port of the S-function (via the MATLAB S-function Api)
*/
static void tptInitInputPort(SimStruct* S, int portNum, tpt_sl_port_data *mlPortData, tpt_port_data * tptPortData, tpt_bool useDirectFeedThrough){
  
    if(tptPortData->signals[0].nDims >= 2 && tptPortData->signals[0].dims[0] < DIMS_STRUCT_ELEM && tptPortData->signals[0].dims[1] < DIMS_STRUCT_ELEM) {
      /* matrix */
      int_T dims[2];
      DECL_AND_INIT_DIMSINFO(di);
      di.numDims = 2;/* invert dimensions */
      dims[0] = tptPortData->signals[0].dims[0];
      dims[1] = tptPortData->signals[0].dims[1];
      di.dims = dims;
      di.width = dims[0] * dims[1];
      ssSetInputPortDimensionInfo(S, portNum, &di);

    }else if(tptPortData->signals[0].nDims >= 1 && tptPortData->signals[0].dims[0] < DIMS_STRUCT_ELEM) {
      ssSetInputPortWidth(S, portNum, tptPortData->signals[0].dims[0]);
    }else if(tptPortData->signals[0].nDims == 0 || tptPortData->signals[0].dims[0] == DIMS_STRUCT_ELEM) {
      /* scalar */
      ssSetInputPortWidth(S, portNum, 1);
    }
    if(useDirectFeedThrough) {
      ssSetInputPortDirectFeedThrough(S, portNum, 1);
    } else {
      ssSetInputPortDirectFeedThrough(S, portNum, 0);
    }
    
    if(tptPortData->isNonVirtualBus){
      ssSetBusInputAsStruct(S, portNum, 1);
    }
  
    ssSetInputPortComplexSignal(S, portNum, COMPLEX_NO); /* complex signals not supported */
    
    ssSetInputPortRequiredContiguous(S, portNum, 1); /* direct input signal access*/
}

/**
* Initialize an output port of the S-function (via the MATLAB S-function Api)
*/
static void tptInitOutputPort(SimStruct* S, int portNum, tpt_sl_port_data *mlPortData, tpt_port_data * tptPortData){
  if(tptPortData->signals[0].nDims >= 2 && tptPortData->signals[0].dims[0] < DIMS_STRUCT_ELEM && tptPortData->signals[0].dims[1] < DIMS_STRUCT_ELEM) {
    /* matrix on port root level */
    int_T dims[2];
    DECL_AND_INIT_DIMSINFO(di);
    di.numDims = 2;
    dims[0] = tptPortData->signals[0].dims[0];
    dims[1] = tptPortData->signals[0].dims[1];
    di.dims = dims;
    di.width = dims[0] * dims[1];
    ssSetOutputPortDimensionInfo(S, portNum, &di);
  } else if(tptPortData->signals[0].nDims >= 1 && tptPortData->signals[0].dims[0] < DIMS_STRUCT_ELEM) {
    /* array on port root level */
    ssSetOutputPortWidth(S, portNum, tptPortData->signals[0].dims[0]);
  } else if(tptPortData->signals[0].nDims == 0 || tptPortData->signals[0].dims[0] == DIMS_STRUCT_ELEM) {
    /* scalar on port root level */
    ssSetOutputPortWidth(S, portNum, 1);
  }
  
  if(ssIsDataTypeABus(S, mlPortData->portType) == 1){
    ssSetBusOutputObjectName(S, portNum, (char *)ssGetDataTypeName(S,mlPortData->portType));
    if(tptPortData->isNonVirtualBus){
      ssSetBusOutputAsStruct(S, portNum, 1);
    }
  }
  ssSetOutputPortComplexSignal(S, portNum, COMPLEX_NO); /* complex signals not supported */
}

/**
* Initializes all internal port data for function call signals
*/
static void tptInitFcnCallsPortData(SimStruct* S, const char* fcnCallSignalNames[], const tpt_tsize fcnCallSignalDims[], int nFcnCalls, tpt_sl_port_data *mlPortData, tpt_port_data * outTptPortData, 
    tpt_bool **fcnCallStates,  tpt_bool **fcnCallStatesOld, tpt_tsize *outTotalFcnCallsDim){
  int i;
  tpt_tsize totalDim;

  mlPortData->portType = SS_FCN_CALL;
  outTptPortData->isNonVirtualBus = tpt_false;
  outTptPortData->nPortSignals = nFcnCalls;
  outTptPortData->signals = tpt_sfun_malloc(nFcnCalls * sizeof(tpt_port_signal));
  mlPortData->nLeafElems = nFcnCalls;
  mlPortData->leafElemMlTypes = tpt_sfun_malloc(nFcnCalls * sizeof(DTypeId));
  outTptPortData->writeInitialValue = tpt_true; /* write initial value to be able to detect trigger at t==0 */

  totalDim = 0;
  for(i=0; i < nFcnCalls; i++){
    /* allways allocate new memory and copy the name, since we allways free the memory for the name later on */
    char* sigNameCpy = tpt_sfun_malloc(sizeof(char)*((tpt_uint32)strlen(fcnCallSignalNames[i])+1));
    strcpy(sigNameCpy, fcnCallSignalNames[i]);
    outTptPortData->signals[i].name = sigNameCpy;
    outTptPortData->signals[i].isScaledFixedPointType = tpt_false;
    outTptPortData->signals[i].dataType = tpt_type_bool;
    outTptPortData->signals[i].nSlTypes = 1;
    outTptPortData->signals[i].slTypeIds = tpt_sfun_malloc(sizeof(tpt_uint32));/* type ids stored as uint32 in tpt-internal data */
    outTptPortData->signals[i].slTypeIds[0] = SS_BOOLEAN; /* use boolean data type for function-calls in TPT */
    outTptPortData->signals[i].slTypeSizes = tpt_sfun_malloc(sizeof(tpt_uint32));
    outTptPortData->signals[i].slTypeSizes[0] = sizeof(tpt_bool);
    
    outTptPortData->signals[i].mlBusElementOffset = totalDim*sizeof(tpt_bool); /* offset within local array */
    if(fcnCallSignalDims[i]>1) {
      totalDim = totalDim + fcnCallSignalDims[i];
      outTptPortData->signals[i].nDims = 1;
      outTptPortData->signals[i].dims =  tpt_sfun_malloc(1 * sizeof(tpt_tsize));
      outTptPortData->signals[i].dims[0] = fcnCallSignalDims[i];
    } else {
      totalDim = totalDim + 1;
      outTptPortData->signals[i].nDims = 0;
      outTptPortData->signals[i].dims = TPT_NULL;
    }
    /* elements-offsets not needed since not within struct */
    outTptPortData->signals[i].nMlElementOffsets = 0;
    outTptPortData->signals[i].mlElementOffsets = TPT_NULL;
    mlPortData->leafElemMlTypes[i] = SS_FCN_CALL;
  }
  /* set number of scalar function call signals */
  *outTotalFcnCallsDim = totalDim;
  *fcnCallStates = (tpt_bool*) tpt_sfun_malloc(totalDim * sizeof(tpt_bool));
  *fcnCallStatesOld = (tpt_bool*) tpt_sfun_malloc(totalDim * sizeof(tpt_bool));
  outTptPortData->portBaseAddress = *fcnCallStates; /* set local memory as output adddress for function data, will be evaluated in mdlOutputs to trigger fcn-calls */
  
  outTptPortData->dataInitialized = tpt_true;
}
  
/**
 * Assign addresses of MATLAB input port signals to tptPortData structure
*/
static void tptAssignInputPortSignalAddresses(SimStruct* S, tpt_in_out* tptIO) {
  int_T iPort = 0;
  for(iPort=0; iPort < tptIO->tpt_ins_size; iPort++){
    tptIO-> tpt_ins_data[iPort].portBaseAddress = (void *) ssGetInputPortSignal(S, iPort);
  }
}

/**
 * Assign addresses of MATLAB output port signals to tptPortData structure
*/
static void tptAssignOutputPortSignalAddresses(SimStruct* S, tpt_in_out* tptIO, tpt_bool firstOutIsFcnCall) {
  int_T iPort = 0;
  if (firstOutIsFcnCall) {
	  iPort++; /* skip first port for assigning addresses as this is handled differently */
  }
  for(; iPort < tptIO->tpt_outs_size; iPort++){
    tptIO-> tpt_outs_data[iPort].portBaseAddress = ssGetOutputPortSignal(S, iPort);
  }
}

#ifdef TPT_SFUN_FRAME
/* fixdt poer api functions only for main TPT S-Function */
 
/**
 * Copy value from the TPT VM to Simulink and apply scaling, called from the inner part of the S-function
*/
static void tptCopyOutportFixdtDataTptToSl(void *simStructPtr, tpt_uint32 portIdx, tpt_uint32 elemIdx, 
  void *tptVMAddress, tpt_basetype vmType, tpt_uint8 vmByteSize, 
  void *mlAddress, tpt_uint32 mlTypeSize){
  fxpOverflowLogs fxpLogs;
  tpt_sl_port_data *portData;
  DTypeId mlType;
  DTypeId vmTypeMlTypeId;

  vmTypeMlTypeId = getMlBuiltInType(vmType); /* ML type id for type from VM */
  portData = &tpt_out_sl_port_data[portIdx]; 
  mlType = portData->leafElemMlTypes[elemIdx];

  ssFxpConvert((SimStruct *)simStructPtr,
    mlAddress, mlTypeSize, mlType, 
    tptVMAddress, vmByteSize, vmTypeMlTypeId,
    portData->fixptRoundingMode, portData->fixptOverflowMode, &fxpLogs);
}

/**
 * Copy value from Simulink to the TPT VM and apply scaling, called from the inner part of the S-function
*/
static void tptCopyInportFixdtDataSlToTpt(void *simStructPtr, tpt_uint32 portIdx, tpt_uint32 elemIdx, 
  void *tptVMAddress, tpt_basetype vmType, tpt_uint8 vmByteSize, 
  void *mlAddress, tpt_uint32 mlTypeSize){
  fxpOverflowLogs fxpLogs;
  tpt_sl_port_data * portData;
  DTypeId mlType;
  DTypeId vmTypeMlTypeId;
          
  vmTypeMlTypeId = getMlBuiltInType(vmType); /* ML type id for type from VM */
  portData = &tpt_in_sl_port_data[portIdx]; 
  mlType = portData->leafElemMlTypes[elemIdx];

  ssFxpConvert((SimStruct *)simStructPtr, 
    tptVMAddress, vmByteSize, vmTypeMlTypeId,
    mlAddress, mlTypeSize, mlType, 
    portData->fixptRoundingMode, portData->fixptOverflowMode, &fxpLogs);
}
#endif /* TPT_SFUN_FRAME */

#ifdef TPT_USE_SL_FCN_STUB
/* *** support for simulink functions *** */

/**
 * Copy value from the TPT VM to Simulink and apply scaling, called from the inner part of the S-function
*/
static void tptCopyFcnOutArgFixdtDataTptToSl(void *simStructPtr, tpt_int32 serverFcnIdx,  tpt_uint32 portIdx, tpt_uint32 elemIdx, 
  void *tptVMAddress, tpt_basetype vmType, tpt_uint8 vmByteSize, 
  void *mlAddress, tpt_uint32 mlTypeSize){
  fxpOverflowLogs fxpLogs;
  tpt_sl_port_data *portData;
  DTypeId mlType;
  DTypeId vmTypeMlTypeId;

  vmTypeMlTypeId = getMlBuiltInType(vmType); /* ML type id for type from VM */
  portData = &tpt_server_sl_function_data[serverFcnIdx].outputArgsData[portIdx].argData;
  mlType = portData->leafElemMlTypes[elemIdx];
  
  ssFxpConvert((SimStruct *)simStructPtr,
    mlAddress, mlTypeSize, mlType, 
    tptVMAddress, vmByteSize, vmTypeMlTypeId,
    portData->fixptRoundingMode, portData->fixptOverflowMode, &fxpLogs);
}

/**
 * Copy value from Simulink to the TPT VM and apply scaling, called from the inner part of the S-function
*/
static void tptCopyFcnInArgFixdtDataSlToTpt(void *simStructPtr, tpt_int32 serverFcnIdx, tpt_uint32 portIdx, tpt_uint32 elemIdx, 
  void *tptVMAddress, tpt_basetype vmType, tpt_uint8 vmByteSize, 
  void *mlAddress, tpt_uint32 mlTypeSize){
  fxpOverflowLogs fxpLogs;
  tpt_sl_port_data * portData;
  DTypeId mlType;
  DTypeId vmTypeMlTypeId;

  vmTypeMlTypeId = getMlBuiltInType(vmType); /* ML type id for type from VM */
  portData = &tpt_server_sl_function_data[serverFcnIdx].inputArgsData[portIdx].argData;
  mlType = portData->leafElemMlTypes[elemIdx];

  ssFxpConvert((SimStruct *)simStructPtr, 
    tptVMAddress, vmByteSize, vmTypeMlTypeId,
    mlAddress, mlTypeSize, mlType, 
    portData->fixptRoundingMode, portData->fixptOverflowMode, &fxpLogs);
}

static void tptInitFunctionArg(SimStruct* S, tpt_sl_function_data *slFcnData, tpt_sl_function_arg_data *slFcnArgData, int dim1, int dim2, tpt_port_data *tptFcnArgData){
  /* concat function and argument name to internal signal name */
  char *argFullName = tpt_sfun_malloc(sizeof(char)*(strlen(slFcnData->name)+strlen(slFcnArgData->name)+2)); /* <fcn>:<argName> */
  sprintf(argFullName, "%s:%s", slFcnData->name, slFcnArgData->name);
  /* init internal data */
  tptInitTptPortData(S, argFullName, &slFcnArgData->argData, dim1, dim2, tpt_false, tptFcnArgData);
  tpt_sfun_free(argFullName);

  /* now initialize the Simulink side */
  ssSetSimulinkFunctionArgDataType(S, slFcnData->name, slFcnArgData-> name, slFcnArgData->argData.portType);
  
  if(tptFcnArgData->signals[0].nDims >= 2 && tptFcnArgData->signals[0].dims[0] < DIMS_STRUCT_ELEM && tptFcnArgData->signals[0].dims[1] < DIMS_STRUCT_ELEM) {
    /* matrix on argument root level */
    size_t dims[2];
    dims[0] = tptFcnArgData->signals[0].dims[0];
    dims[1] = tptFcnArgData->signals[0].dims[1];
    ssSetSimulinkFunctionArgDimensions(S, slFcnData->name, slFcnArgData-> name, 2, dims);
  } else if(tptFcnArgData->signals[0].nDims >= 1 && tptFcnArgData->signals[0].dims[0] < DIMS_STRUCT_ELEM) {
    /*array on argument root level */
    size_t dims[1];
    dims[0] = tptFcnArgData->signals[0].dims[0];
    ssSetSimulinkFunctionArgDimensions(S, slFcnData->name, slFcnArgData-> name, 1, dims);
  } else if(tptFcnArgData->signals[0].nDims == 0 || tptFcnArgData->signals[0].dims[0] == DIMS_STRUCT_ELEM) {
    /* scalar on argument root level */
    size_t dims[1] = {1};
    ssSetSimulinkFunctionArgDimensions(S, slFcnData->name, slFcnArgData-> name, 1, dims);
  }
  
  ssSetSimulinkFunctionArgComplexity(S, slFcnData->name, slFcnArgData-> name, 0);
}

static void tptAssignSimulinkFunctionArgumentAddresses(SimStruct* S, ssFcnCallExecArgs *args, tpt_fcn_data *tptServerFcnData){
  int iArg;
  for(iArg = 0; iArg < tptServerFcnData->nFcnInArgs; iArg++){
    tptServerFcnData->fcnInArgs[iArg].portBaseAddress = ssGetSimulinkFunctionInput(args, iArg);
  }
  for(iArg = 0; iArg < tptServerFcnData->nFcnOutArgs; iArg++){
    tptServerFcnData->fcnOutArgs[iArg].portBaseAddress = ssGetSimulinkFunctionOutput(args, iArg);
  }
}

static void tptCleanupFcnInterfaceData(const tpt_server_functions *fcn_data, tpt_sl_function_data *slServerFunctionData) {
  int iFcn, iFcnArg;
  for(iFcn = 0; iFcn < fcn_data->tpt_server_fcns_size; iFcn++){
    for(iFcnArg = 0; iFcnArg < fcn_data->tpt_server_fcns_data[iFcn].nFcnInArgs; iFcnArg++){
      cleanupPortData(&fcn_data->tpt_server_fcns_data[iFcn].fcnInArgs[iFcnArg], &slServerFunctionData[iFcn].inputArgsData[iFcnArg].argData);
    }
    for(iFcnArg = 0; iFcnArg < fcn_data->tpt_server_fcns_data[iFcn].nFcnOutArgs; iFcnArg++){
      cleanupPortData(&fcn_data->tpt_server_fcns_data[iFcn].fcnOutArgs[iFcnArg], &slServerFunctionData[iFcn].outputArgsData[iFcnArg].argData);
    }
  }
}

#endif /* TPT_USE_SL_FCN_STUB */
