package com.etas.ascet.api.v1;

import java.util.List;
import java.util.concurrent.CompletableFuture;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.lsp4j.jsonrpc.services.JsonRequest;
import org.eclipse.lsp4j.jsonrpc.services.JsonSegment;

import com.etas.ascet.api.AscetApiGeneric;
import com.etas.ascet.api.AscetDetailsDto;
import com.etas.ascet.api.KeyValueDto;
import com.etas.ascet.api.v1.datatype.AscetCodegeneratorDetailListDtoV1;
import com.etas.ascet.api.v1.datatype.LaunchCopyRequestDtoV1;
import com.etas.ascet.api.v1.datatype.LaunchCreateWithDefaultsRequestDtoV1;
import com.etas.ascet.api.v1.datatype.LaunchDetailDtoV1;
import com.etas.ascet.api.v1.datatype.LaunchExecuteResultDtoV1;
import com.etas.ascet.api.v1.datatype.LaunchSetValuesRequestDtoV1;
import com.etas.ascet.api.v1.datatype.TargetClassTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetGetContentResponseDtoV1;
import com.etas.ascet.api.v1.datatype.WorkspaceGetOverviewResponseDtoV1;

/**
 * API version 1 for ASCET-DEVELOPER.
 * Use this interface in pair with {@link ClientApiV1}.
 * <p/>
 * The method signature schema is:
 *
 * <pre>
 * CompletableFuture&lt;result-type> &lt;method-name>( &lt;arguments...> )
 * &lt;method-name> ::= &lt;api-version>&lt;scope>&lt;action-name>
 * </pre>
 *
 * The reason behind the verbose method name is, that all available interfaces that are supported at runtime,
 * are mapped into a single proxy object.
 * This is because of the technology of JSON-RCP implementation.
 * In this moment, all the methods of all interfaces are not allowed to have
 * name collisions. Hence there is only a single interface for each version,
 * which prefixes all methods with the version and the scope (instead of having distinct one interface per scope).
 * <p/>
 * Scope:
 * <ul>
 * <li><b>Program:</b> ASCET-DEVELOPER instance methods. Those are to control the running ASCET program.
 * It is not about the user's content.</li>
 * <li><b>Application:</b> the ASCET application (.app) files</li>
 * <li><b>Model:</b> model information, as it is written in ESDL.
 * Each class exists once as it exists in the source code.
 * Representation information is not yet resolved.
 * Instance information is not yet available.</li>
 * <li><b>Target:</b> In combination with application and a {@link TargetConfigDtoV1} the code generator can run and
 * build the target output.
 * Or the model, transformed to target-view can be queried.
 * In the target view, the model has the representations resolved.
 * A {@link TargetClassTypeDtoV1} can exists multiple times for the same ESDL class,
 * once per representation in use.</li>
 * <li><b>Workspace:</b> the Eclipse/ASCET-DEVELOPER workspace, projects, files in the project-navigator view.</li>
 * </ul>
 */
@NonNullByDefault
@JsonSegment("com.etas.ascet.api.server-v1")
public interface AscetApiV1 {

    // ---------------------------------------------------------------------
    // ---- PROGRAM --------------------------------------------------------
    // ASCET-DEVELOPER itself

    /**
     * @return the version of ASCET-DEVELOPER tool. E.g. "7.8.0"
     * @since 1.0
     */
    @JsonRequest
    CompletableFuture<AscetDetailsDto>
            v1ProgramGetAscetDetails();

    /**
     * Close the running ASCET-DEVELOPER instance. The response is returned and ASCET-DEVELOPER is closed after a short
     * delay.
     *
     * @return <code>true</code> if the application is going to close,
     *         <code>false</code> if the preconditions are not met, e.g.
     *         dirty editors, running jobs or open dialogs
     * @since 1.0
     */
    @JsonRequest
    CompletableFuture<Boolean>
            v1ProgramCloseAscet();

    /**
     * Restart the running ASCET-DEVELOPER instance.
     * The response is returned and ASCET-DEVELOPER is restarted after a short
     * delay. The connection will be closed during the restart and the client needs to take care to reestablish a new
     * connection.
     *
     * @param path
     *            absolute path to the new workspace location.
     * @return <code>true</code> if the application is going to restart,
     *         <code>false</code> if the preconditions are not met, e.g.
     *         dirty editors, running jobs or open dialogs
     * @since 1.0
     */
    @JsonRequest
    CompletableFuture<Boolean>
            v1ProgramSwitchWorkspace(String path);

    /**
     * @return a list of installed code genertor servers
     */
    @JsonRequest
    CompletableFuture<AscetCodegeneratorDetailListDtoV1>
            v1ProgramCodegeneratorDetailList();

    // ---------------------------------------------------------------------
    // ---- TARGET ---------------------------------------------------------
    // the target view of the model (all representation resolved)

    /**
     * Collect items from the instance tree and about the used types.
     * <p/>
     * The workspace must not have active error markers.
     * <p/>
     * The launch type must be a code generation build type.
     * E.g. a EHandbook build will not work.
     * <p/>
     * Before collecting the information, an incremental build is started.
     * This may take a significant amount of execution time.
     *
     * @param launchName
     *            The launch configuration of type ECU code generation,
     *            PC Experiment or Rapid Prototyping.
     * @return the list of items found
     * @throws IllegalStateException
     *             if the workspace contains errors.
     * @since 1.0
     */
    @JsonRequest
    CompletableFuture<TargetGetContentResponseDtoV1>
            v1TargetGetContent(String launchName);

    // ---------------------------------------------------------------------
    // MODEL --------------------------------------------------------------
    // the model view as it is written in ESDL

    // ---------------------------------------------------------------------
    // LAUNCH --------------------------------------------------------------
    // launch configurations

    /**
     * Creates a launch configuration, that is visible in the common launch (run) dialog.
     * The name must be unique, if it already exists, a unique name is created and returned.
     * Use the returned name to refer to this launch configuration.
     * Take care, that the launch name is a shared resource.
     * Parallel connections might conflict/overlap with the name.
     * The launch type must be a supported launch type in the server.
     *
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<String>
            v1LaunchCreateWithDefaults(LaunchCreateWithDefaultsRequestDtoV1 request);

    /**
     * Creates a launch configuration, that is visible in the common launch (run) dialog.
     * The name must be unique, if it already exists, a unique name is created and returned.
     * Use the returned name to refer to this launch configuration.
     * Take care, that the launch name is a shared resource.
     * Parallel connections might conflict/overlap with the name.
     * The launch type must be a supported launch type in the server.
     *
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<String>
            v1LaunchCopy(LaunchCopyRequestDtoV1 request);

    /**
     * Read all values from the launch configuration.
     *
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<List<KeyValueDto>>
            v1LaunchGetValues(String launchName);

    /**
     * Modify value in an existing launch configuration
     *
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<Void>
            v1LaunchSetValues(LaunchSetValuesRequestDtoV1 request);

    /**
     * Delete an existing launch configuration
     *
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<Void>
            v1LaunchDelete(String launchName);

    /**
     * Execute an existing launch configuration
     *
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<LaunchExecuteResultDtoV1>
            v1LaunchExecute(String launchName);

    // WORKSPACE -------------------------------------------------------------
    // the (Eclipse/Ascet) workspace
    /**
     * Collect a list of all ASCET applications that are present in the workspace.
     * <p/>
     * Before collecting the information, an incremental build is started.
     * This may take a significant amount of execution time.
     *
     * @return all applications in the workspace, as workspace paths
     * @throws IllegalStateException
     *             if the workspace contains errors.
     * @since 1.0
     */
    @JsonRequest
    CompletableFuture<List<String>>
            v1WorkspaceGetApplications();

    /**
     * Find all launches in the workspace. That is all entries in the "Run ..." dialog tree.
     * Each list entry contains the name and the workspace path.
     * The later is optional, because not all launches are stored to file.
     *
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 3 LINES
    @JsonRequest
    CompletableFuture<List<LaunchDetailDtoV1>>
            v1WorkspaceGetLaunches();

    /**
     * @return details about workspace and projects. Same like {@link AscetApiGeneric#genericGetAscetDetails()}
     * @since 1.0
     */
    @JsonRequest
    CompletableFuture<WorkspaceGetOverviewResponseDtoV1>
            v1WorkspaceGetOverview();

    /**
     * Import the project at 'path'
     *
     * @param path
     *            a Path to a folder containing a '.project' file.
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<Void>
            v1WorkspaceImportProject(String path);

    /**
     * @param projectName
     *            the name of the project to delete.
     * @since 1.0
     */
    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 5 LINES
    @JsonRequest
    CompletableFuture<Void>
            v1WorkspaceDeleteProject(String projectName);

}
