package com.etas.ascet.api.v1;

import static java.util.stream.Collectors.*;
import static org.apache.commons.lang3.Validate.*;

import java.util.List;

import org.apache.commons.lang3.Validate;
import org.eclipse.jdt.annotation.NonNullByDefault;

import com.etas.ascet.api.v1.datatype.IdentifierDtoV1;
import com.etas.ascet.api.v1.datatype.TargetArrayTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetAxisTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetClassTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetDataInterfaceTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetEnumTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetScalarTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetStructTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetTableTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TargetTypeDtoV1;
import com.etas.ascet.api.v1.datatype.TypeKindV1;

/**
 * Converts concrete types into the generic TargetTypeDtoV1.
 */
@NonNullByDefault
public final class TypeDtoUtils {
    private TypeDtoUtils() {
        // nothing
    }

    // CHECKSTYLE IGNORE JavadocMethod FOR NEXT 200 LINES

    public static TargetTypeDtoV1 from(final TargetScalarTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.SCALAR)
                .scalarType(type)
                .build();
    }

    public static TargetTypeDtoV1 from(final TargetClassTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.CLASS)
                .classType(type)
                .build();
    }

    public static TargetTypeDtoV1 from(final TargetStructTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.STRUCT)
                .structType(type)
                .build();
    }

    public static TargetTypeDtoV1 from(final TargetDataInterfaceTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.DATA_INTERFACE)
                .dataInterfaceType(type)
                .build();
    }

    public static TargetTypeDtoV1 from(final TargetEnumTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.ENUM)
                .enumType(type)
                .build();
    }

    public static TargetTypeDtoV1 from(final TargetArrayTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.ARRAY)
                .arrayType(type)
                .build();
    }

    public static TargetTypeDtoV1 from(final TargetTableTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.TABLE)
                .tableType(type)
                .build();
    }

    public static TargetTypeDtoV1 from(final TargetAxisTypeDtoV1 type) {
        return TargetTypeDtoV1.builder()
                .identifier(type.identifier())
                .kind(TypeKindV1.AXIS)
                .axisType(type)
                .build();
    }

    public static TargetScalarTypeDtoV1 getScalar(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.SCALAR);
        return notNull(type.scalarType());
    }

    public static TargetClassTypeDtoV1 getClass(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.CLASS);
        return notNull(type.classType());
    }

    public static TargetStructTypeDtoV1 getStruct(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.STRUCT);
        return notNull(type.structType());
    }

    public static TargetDataInterfaceTypeDtoV1 getDataInterface(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.DATA_INTERFACE);
        return notNull(type.dataInterfaceType());
    }

    public static TargetEnumTypeDtoV1 getEnum(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.ENUM);
        return notNull(type.enumType());
    }

    public static TargetArrayTypeDtoV1 getArray(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.ARRAY);
        return notNull(type.arrayType());
    }

    public static TargetTableTypeDtoV1 getTable(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.TABLE);
        return notNull(type.tableType());
    }

    public static TargetAxisTypeDtoV1 getAxis(final TargetTypeDtoV1 type) {
        isTrue(type.kind() == TypeKindV1.AXIS);
        return notNull(type.axisType());
    }

    public static List<TargetScalarTypeDtoV1> extractScalars(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.SCALAR)
                .map(TargetTypeDtoV1::scalarType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static List<TargetClassTypeDtoV1> extractClasses(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.CLASS)
                .map(TargetTypeDtoV1::classType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static List<TargetStructTypeDtoV1> extractStructs(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.STRUCT)
                .map(TargetTypeDtoV1::structType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static List<TargetDataInterfaceTypeDtoV1> extractDataInterfaces(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.DATA_INTERFACE)
                .map(TargetTypeDtoV1::dataInterfaceType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static List<TargetEnumTypeDtoV1> extractEnums(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.ENUM)
                .map(TargetTypeDtoV1::enumType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static List<TargetArrayTypeDtoV1> extractArrays(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.ARRAY)
                .map(TargetTypeDtoV1::arrayType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static List<TargetTableTypeDtoV1> extractTables(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.TABLE)
                .map(TargetTypeDtoV1::tableType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static List<TargetAxisTypeDtoV1> extractAxes(final List<TargetTypeDtoV1> types) {
        return types.stream()
                .filter(t -> t.kind() == TypeKindV1.AXIS)
                .map(TargetTypeDtoV1::axisType)
                .map(Validate::notNull)
                .collect(toList());
    }

    public static TargetTypeDtoV1 getByIdentifier(final List<TargetTypeDtoV1> types, final IdentifierDtoV1 identifier) {
        return types.stream()
                .filter(type -> type.identifier().equals(identifier))
                .findFirst()
                .orElseThrow();
    }

    public static TargetClassTypeDtoV1 getClassByIdentifier(final List<TargetTypeDtoV1> types,
            final IdentifierDtoV1 identifier) {
        return notNull(getByIdentifier(types, identifier).classType());
    }
}
