/* RegistryValue.java
 *
 * jRegistryKey - A JNI wrapper of the Windows Registry functions.
 * Copyright (c) 2001, BEQ Technologies Inc.
 * 9407 - 27 Avenue
 * Edmonton, Alberta
 * T6N - 1L9 Canada
 * (780) 430-0056
 * (780) 437-6121 (fax)
 * http://www.beq.ca
 *
 * Original Author: Joe Robinson <joe@beq.ca>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ca.beq.util.win32.registry;

import java.io.Serializable;

/**
 * A representation of registry values. A registry value is defined as a
 * collection of the following properties: <code>name</code>,
 * <code>type</code>, and <code>data</code>.
 * 
 * <p>
 * A <code>name</code> consists of a string of one or more printable
 * characters, excluding the backslash (\). A <code>type</code> may be any of
 * the defined <code>ValueType</code>'s. <code>Data</code> is any object,
 * consistent with the <code>ValueType</code>.
 * 
 * @author BEQ Technologies Inc.
 * @version $Revision: 1.16 $
 * 
 * <pre>
 * $Id: RegistryValue.java,v 1.16 2006/07/10 01:38:56 bayblade Exp $
 * 
 * $Log: RegistryValue.java,v $
 * Revision 1.16  2006/07/10 01:38:56  bayblade
 * Formatting
 * 
 * Revision 1.15  2006/07/06 04:30:28  bayblade
 * Adding a specific equals() method to compare the actual value and type, courtesy Dave Kipfer &lt;kipfer@cae.com&gt;
 * 
 * Revision 1.14  2006/06/11 16:46:53  bayblade
 * Introducing Win32Exception ...
 * Subclassed RegistryException with new Win32Exception that contains the windows error message, and a method to see the error code as an int rather than a formatted string containing the error code.  The exception message is different, which may break logic that parses this message, but explicitly catching the subclass now allows for greater refinement when processing what actually broke, and is a better solution overall.
 * 
 * Revision 1.13  2006/03/02 03:24:25  bayblade
 * Sanity checks for null value types in both the Java and native code, courtesy Arthur Shagall
 * 
 * Revision 1.11  2005/12/05 03:22:59  bayblade
 * Hacked to accomodate the versioning sync between the jni and the java portions of the code.  Note that a version mismatch on implicit calls will spew data on stderr, and a mismatch on an explicit call will throw a Registry exception, although initialization will have occured anyhow.  As this is error, not common behavior, its not expected to break anything that's not broken anyhow.
 * 
 * Revision 1.10  2005/05/10 01:01:48  bayblade
 * 3 things happened here: added support for reg_qword, support for reg_multi_sz and finally unhooped unicode character conversion.
 * 
 * Revision 1.9  2005/01/25 22:38:11  bayblade
 * Most noticably, the fix for 1105000 which both reworks the Unicode to UTF-8 mechanism and permits REG_SZ data to run through the system as byte[] (though byte[] defaults to REG_BINARY)
 * 
 * Revision 1.8  2004/12/20 21:21:31  bayblade
 * Revampled initialization process so 1.0 upgraders weren't so confused;  fixed bug with IBM machines; and accidentally reverted to ANSI formatting instead of K&amp;R in a few spots
 * 
 * Revision 1.7  2004/04/20 00:16:18  bayblade
 * Added &lt;pre&gt; tags in headers so change history log 
 * can be sensibly read from javadoc
 * 
 * Revision 1.6  2004/04/13 19:17:48  mollinie
 * Call to constructors of the class will fail unless initialization succeeded (in order
 * to enforce the initialize-before-use policy).
 * Corrected some Javadoc.
 * 
 * Revision 1.5  2004/04/07 18:58:26  mollinie
 * Removed useless 'this' pointers.
 * 
 * Revision 1.4  2004/04/07 18:24:55  mollinie
 * Added poor man's support for extra registry data types, including REG_MULTI_SZ, REG_LINK, REG_FULL_RESOURCE_DESCRIPTOR and REG_RESOURCE_REQUIREMENTS_LIST.
 * 
 * Revision 1.3  2004/03/27 01:01:01  mollinie
 * Preliminary reformating.
 * 
 * Revision 1.2  2004/03/27 00:39:23  mollinie
 * Preliminary reformating.
 * 
 * Revision 1.1.1.1  2004/03/25 22:05:38  bayblade
 * Initial Import
 * </pre>
 */

public class RegistryValue implements Serializable {

	/*
	 * This may be hypersensitive to versioning changes, VM changes, phases of
	 * the moon and all kinds of verbal abuse, but hey--its Serialization, its
	 * not like using Serialization hasn't earned you some pain (it has, btw)
	 */
	private static final long serialVersionUID = RegistryValue.class.hashCode()
			* Version.current.hashCode();

	// implicit initialization. see method for details.
	static {
		try {
			RegistryKey.initialize();
		} catch (RegistryException re) {
			re.printStackTrace();
		}
	}

	String m_name;

	ValueType m_type;

	Object m_data;

	/**
	 * Constructs a new, empty, <code>RegistryValue</code>.
	 */
	public RegistryValue() {
		m_name = "";
		m_type = ValueType.REG_SZ;
		m_data = null;
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified data.
	 * The type is defaulted to <code>ValueType.REG_SZ</code>. The
	 * <code>name</code> defaults to ""; the default value name.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(Object data) {
		this();
		m_data = data;
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name and
	 * data. The type is defaulted to <code>ValueType.REG_SZ</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, Object data) {
		this();
		m_name = name;
		m_data = data;
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * type, and data.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param type
	 *            the <code>RegistryValue</code>'s type
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, ValueType type, Object data) {
		this();
		if (type == null) {
			throw new NullPointerException("ValueType cannot be null");
		}
		m_name = name;
		m_type = type;
		m_data = data;
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * and data. The type is <code>ValueType.REG_BINARY</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, byte[] data) {
		this(name, ValueType.REG_BINARY, data);
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * and data. The type is <code>ValueType.REG_DWORD</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, boolean data) {
		this(name, ValueType.REG_DWORD, new Boolean(data));
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * and data. The type is <code>ValueType.REG_BINARY</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, byte data) {
		this(name, ValueType.REG_BINARY, new Byte(data));
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * and data. The type is <code>ValueType.REG_DWORD</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, int data) {
		this(name, ValueType.REG_DWORD, new Integer(data));
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * and data. The type is <code>ValueType.REG_QWORD</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, long data) {
		this(name, ValueType.REG_QWORD, new Long(data));
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * and data. The type is <code>ValueType.REG_BINARY</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, float data) {
		this(name, ValueType.REG_BINARY, new Float(data));
	}

	/**
	 * Constructs a new <code>RegistryValue</code> with the specified name,
	 * and data. The type is <code>ValueType.REG_BINARY</code>.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public RegistryValue(String name, double data) {
		this(name, ValueType.REG_BINARY, new Double(data));
	}

	/**
	 * Returns this <code>RegistryValue</code>'s name.
	 * 
	 * @return the <code>RegistryValue</code>'s name
	 */
	public String getName() {
		return m_name;
	}

	/**
	 * Sets this <code>RegistryValue</code>'s name.
	 * 
	 * @param name
	 *            the <code>RegistryValue</code>'s name
	 */
	public void setName(String name) {
		m_name = name;
	}

	/**
	 * Returns this <code>RegistryValue</code>'s <code>ValueType</code>.
	 * 
	 * @return the <code>RegistryValue</code>'s type
	 */
	public ValueType getType() {
		return m_type;
	}

	/**
	 * Sets this <code>RegistryValue</code>'s <code>ValueType</code>.
	 * 
	 * @param type
	 *            the <code>RegistryValue</code>'s type
	 */
	public void setType(ValueType type) {
		if (type == null) {
			throw new NullPointerException("ValueType cannot be a null");
		}
		m_type = type;
	}

	/**
	 * Returns this <code>RegistryValue</code>'s data.
	 * 
	 * @return the <code>RegistryValue</code>'s data
	 */
	public Object getData() {
		return m_data;
	}

	/**
	 * Sets this <code>RegistryValue</code>'s data.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public void setData(Object data) {
		m_data = data;
	}

	/**
	 * Sets this <code>RegistryValue</code>'s data.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public void setData(byte data) {
		setData(new Byte(data));
	}

	/**
	 * Sets this <code>RegistryValue</code>'s data.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public void setData(boolean data) {
		setData(new Boolean(data));
	}

	/**
	 * Sets this <code>RegistryValue</code>'s data.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public void setData(int data) {
		setData(new Integer(data));
	}

	/**
	 * Sets this <code>RegistryValue</code>'s data.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public void setData(long data) {
		setData(new Long(data));
	}

	/**
	 * Sets this <code>RegistryValue</code>'s data.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public void setData(float data) {
		setData(new Float(data));
	}

	/**
	 * Sets this <code>RegistryValue</code>'s data.
	 * 
	 * @param data
	 *            the <code>RegistryValue</code>'s data
	 */
	public void setData(double data) {
		setData(new Double(data));
	}

	/**
	 * Compares this value with <code>other</code>
	 * 
	 * @param other
	 *            the other <code>RegistryValue</code>
	 */
	public boolean equals(RegistryValue other) {
		if (m_type != other.getType())
			return false;
		if (m_type == ValueType.REG_SZ || m_type == ValueType.REG_EXPAND_SZ)
			return m_data.toString().equals(other.getData().toString());
		if (m_type == ValueType.REG_DWORD_LITTLE_ENDIAN
				|| m_type == ValueType.REG_DWORD_BIG_ENDIAN
				|| m_type == ValueType.REG_DWORD
				|| m_type == ValueType.REG_QWORD)
			return m_data.equals(other.getData());
		if (m_type == ValueType.REG_MULTI_SZ) {
			String[] wMyStrings = (String[]) m_data;
			String[] wOtherStrings = (String[]) other.getData();
			if (wMyStrings.length != wOtherStrings.length)
				return false;
			for (int wCnt = 0; wCnt < wMyStrings.length; wCnt++) {
				if (!wMyStrings[wCnt].equals(wOtherStrings[wCnt]))
					return false;
			}
			return true;
		}
		if (m_type == ValueType.REG_NONE || m_type == ValueType.REG_BINARY
				|| m_type == ValueType.REG_LINK
				|| m_type == ValueType.REG_RESOURCE_LIST
				|| m_type == ValueType.REG_RESOURCE_REQUIREMENTS_LIST
				|| m_type == ValueType.REG_FULL_RESOURCE_DESCRIPTOR) {
			byte[] wMyData = (byte[]) m_data;
			byte[] wOtherData = (byte[]) other.getData();
			if (wMyData.length != wOtherData.length)
				return false;
			for (int wCnt = 0; wCnt < wMyData.length; wCnt++) {
				if (wMyData[wCnt] != wOtherData[wCnt])
					return false;
			}
			return true;
		}
		return false; // Unknown type
	}

	/**
	 * Returns a string representation of the value held by this
	 * <code>RegistryValue</code>.
	 */
	public String getStringValue() {
		if (m_name == null || m_data == null) {
			throw new NullPointerException("Neither name nor data may be null");
		}

		String stringValue;
		if (m_type == ValueType.REG_SZ || m_type == ValueType.REG_EXPAND_SZ) {
			stringValue = m_data.toString();
		} else if (m_type == ValueType.REG_DWORD_LITTLE_ENDIAN
				|| m_type == ValueType.REG_DWORD_BIG_ENDIAN
				|| m_type == ValueType.REG_DWORD) {
			stringValue = ((Integer) m_data).toString();

		} else if (m_type == ValueType.REG_QWORD) {
			stringValue = ((Long) m_data).toString();

		} else if (m_type == ValueType.REG_MULTI_SZ) {
			String[] strings = (String[]) m_data;
			StringBuffer sb = new StringBuffer();
			for (int i = 0; i < strings.length; i++) {
				sb.append('\n').append(strings[i]);
			}
			stringValue = sb.toString();

		} else if (m_type == ValueType.REG_NONE
				|| m_type == ValueType.REG_BINARY
				|| m_type == ValueType.REG_LINK
				|| m_type == ValueType.REG_RESOURCE_LIST
				|| m_type == ValueType.REG_RESOURCE_REQUIREMENTS_LIST
				|| m_type == ValueType.REG_FULL_RESOURCE_DESCRIPTOR) {
			StringBuffer sb = new StringBuffer();
			byte[] b = (byte[]) m_data;
			for (int index = 0; index < b.length; index++) {
				sb.append(' ').append(Byte.toString(b[index]));
			}
			stringValue = sb.toString();
		} else {
			throw new RegistryException("Unrecognized data type: " + m_type);
		}

		return stringValue;
	}

	/**
	 * Returns a string representation of this <code>RegistryValue</code>.
	 * 
	 * @return the <code>RegistryValue</code> as a String
	 * 
	 * @throws NullPointerException
	 *             if the <code>name</code> or <code>data</code> properties
	 *             are null.
	 */
	public String toString() {
		return (m_name + ':' + m_type.toString() + ':' + getStringValue());
	}
}