/* RootKey.java
 *
 * jRegistryKey - A JNI wrapper of the Windows Registry functions.
 * Copyright (c) 2001, BEQ Technologies Inc.
 * 9407 - 27 Avenue
 * Edmonton, Alberta
 * T6N - 1L9 Canada
 * (780) 430-0056
 * (780) 437-6121 (fax)
 * http://www.beq.ca
 *
 * Original Author: Joe Robinson <joe@beq.ca>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ca.beq.util.win32.registry;

/**
 * The Windows operating system defines standard registry keys ("root keys",
 * represented by <code>RootKey</code>) that are always open.
 * <code>RootKey</code> exposes these predefined keys in a
 * <a href="http://developer.java.sun.com/developer/Books/shiftintojava/page1.html#replaceenums">typesafe
 * enum</a> that applications may use as entry points to the registry.
 *
 * <p>The system provides two predefined keys at the root of the registry:
 *
 * <ul>
 * <li><code>HKEY_LOCAL_MACHINE</code>, and
 * <li><code>HKEY_USERS</code>
 * </ul>
 *
 * <p>In addition, the system defines <code>HKEY_CURRENT_CONFIG</code> (a
 * subkey of <code>HKEY_LOCAL_MACHINE</code>), <code>HKEY_CURRENT_USER</code>
 * (a subkey of <code>HKEY_USERS</code>), and <code>HKEY_CLASSES_ROOT</code> (a
 * subkey that merges information from <code>HKEY_LOCAL_MACHINE</code> and
 * <code>HKEY_CURRENT_USER</code>). These registry keys are valid for all
 * Windows implementations of the registry. In addition, other predefined
 * keys have been defined for specific platforms.
 *
 * @author BEQ Technologies Inc.
 * @version $Revision: 1.7 $
 * <pre>
 * $Id: RootKey.java,v 1.7 2005/12/05 03:22:59 bayblade Exp $
 * 
 * $Log: RootKey.java,v $
 * Revision 1.7  2005/12/05 03:22:59  bayblade
 * Hacked to accomodate the versioning sync between the jni and the java portions of the code.  Note that a version mismatch on implicit calls will spew data on stderr, and a mismatch on an explicit call will throw a Registry exception, although initialization will have occured anyhow.  As this is error, not common behavior, its not expected to break anything that's not broken anyhow.
 *
 * Revision 1.6  2004/12/20 21:32:03  bayblade
 * Implemented Phillipe's change request 1029307
 *
 * Revision 1.5  2004/12/20 21:21:31  bayblade
 * Revampled initialization process so 1.0 upgraders weren't so confused;  fixed bug with IBM machines; and accidentally reverted to ANSI formatting instead of K&R in a few spots
 *
 * Revision 1.4  2004/04/20 00:16:18  bayblade
 * Added &lt;pre&gt; tags in headers so change history log 
 * can be sensibly read from javadoc
 *
 * Revision 1.3  2004/04/07 18:58:50  mollinie
 * Renamed member fields according to our conventions.
 *
 * Revision 1.2  2004/03/27 01:01:01  mollinie
 * Preliminary reformating.
 *
 * Revision 1.1.1.1  2004/03/25 22:05:38  bayblade
 * Initial Import
 * </pre>
 */

public final class RootKey {


    // implicit initialization. see method for details.
    static {
        try {
            RegistryKey.initialize();
        } catch ( RegistryException re ) {
            re.printStackTrace();
        }
    }

    /** The root key name displayed by <code>toString</code> */
    final String m_name;

    /** The root key value used by the native Windows registry functions */
    final int m_value;


    /**
     * Constructs a new root key with the specified display name and value.
     *
     * @param name - the display name of the root key
     * @param value - the integer value of the root key
     */
    private RootKey( String name, int value ) {
        m_name = name;
        m_value = value;
    }


    /**
     * Returns the integer value (used by the native Windows registry functions)
     * of the root key.
     *
     * @return the integer value of the root key.
     */
    protected int getValue() {
        return m_value;
    }


    /**
     * Returns the display name of the root key.
     *
     * @return a string representation of the root key.
     */
    public String toString() {
        return m_name;
    }

    /**
     * Registry entries subordinate to this key define types (or classes) of
     * documents and the properties associated with those types. Shell and
     * COM applications use the information stored under this key.  File
     * viewers and user longerface extensions store their OLE class
     * identifiers in <code>HKEY_CLASSES_ROOT</code>, and in-process servers
     * are registered in this key.
     */
    public static final RootKey HKEY_CLASSES_ROOT = new RootKey(
            "HKEY_CLASSES_ROOT", 0x80000000 );
    
    /**
     * Shorthand alias for HKEY_CLASSES_ROOT
     */
    public static final RootKey HKCR = HKEY_CLASSES_ROOT;

    /**
     * Registry entries subordinate to this key define the preferences of the
     * current user. These preferences include the settings of environment
     * variables, data about program groups, colors, printers, network
     * connections, and application preferences.
     */
    public static final RootKey HKEY_CURRENT_USER = new RootKey(
            "HKEY_CURRENT_USER", 0x80000001 );
    
    /**
     * Shorthand alias for HKEY_CURRENT_USER
     */
    public static final RootKey HKCU = HKEY_CURRENT_USER;


    /**
     * Registry entries subordinate to this key define the physical state of
     * the computer, including data about the bus type, system memory, and
     * installed hardware and software. It contains subkeys that hold current
     * configuration data, including Plug and Play information (the Enum
     * branch, which includes a complete list of all hardware that has ever
     * been on the system), network logon preferences, network security
     * information, software-related information (such as server names and
     * the location of the server), and other system information.
     */
    public static final RootKey HKEY_LOCAL_MACHINE = new RootKey(
            "HKEY_LOCAL_MACHINE", 0x80000002 );
    
    /**
     * Shorthand alias for HKEY_LOCAL_MACHINE
     */
    public static final RootKey HKLM = HKEY_LOCAL_MACHINE;


    /**
     * Registry entries subordinate to this key define the default user
     * configuration for new users on the local computer and the user
     * configuration for the current user.
     */
    public static final RootKey HKEY_USERS = new RootKey( "HKEY_USERS",
            0x80000003 );
    
    /**
     * Shorthand alias for HKEY_USERS
     */
    public static final RootKey HKU = HKEY_USERS;
    

    /**
     * Registry entries subordinate to this key contain information about the
     * current hardware profile of the local computer system. The information
     * under <code>HKEY_CURRENT_CONFIG</code> describes only the differences
     * between the current hardware configuration and the standard configuration.
     */
    public static final RootKey HKEY_CURRENT_CONFIG = new RootKey(
            "HKEY_CURRENT_CONFIG", 0x80000005 );
    
    /**
     * Shorthand alias for HKEY_CURRENT_CONFIG
     */
    public static final RootKey HKCC = HKEY_CURRENT_CONFIG;


    /**
     * Registry entries subordinate to this key allow you to access
     * performance data. The data is not actually stored in the registry; the
     * registry functions cause the system to collect the data from its
     * source.
     *
     * <p>NOTE: HKEY_PERFORMANCE_DATA is defined only for Windows NT 4.0,
     * Windows 2000, and Windows XP operating systems.
     */
    public static final RootKey HKEY_PERFORMANCE_DATA = new RootKey(
            "HKEY_PERFORMANCE_DATA", 0x80000004 );
    
    /**
     * Shorthand alias for HKEY_PERFORMANCE_DATA
     */
    public static final RootKey HKPD = HKEY_PERFORMANCE_DATA;


    /**
     * Registry entries subordinate to this key allow you to collect
     * performance data.
     *
     * <p>NOTE: HKEY_DYN_DATA is defined only for Windows 95, Windows 98,
     * and Windows ME operating systems.
     */
    public static final RootKey HKEY_DYN_DATA = new RootKey( "HKEY_DYN_DATA",
            0x80000006 );
    
    /**
     * Alias for HKEY_DYN_DATA
     */
    public static final RootKey HKDD = HKEY_DYN_DATA;
    
}
