/* ValueIterator.java
 *
 * jRegistryKey - A JNI wrapper of the Windows Registry functions.
 * Copyright (c) 2001, BEQ Technologies Inc.
 * 9407 - 27 Avenue
 * Edmonton, Alberta
 * T6N - 1L9 Canada
 * (780) 430-0056
 * (780) 437-6121 (fax)
 * http://www.beq.ca
 *
 * Original Author: Joe Robinson <joe@beq.ca>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ca.beq.util.win32.registry;

import java.util.Iterator;

/**
 * <code>ValueIterator</code> provides methods to iterate through the values
 * of a particular registry key.
 *
 * @see ca.beq.util.win32.registry.RegistryKey
 * @see ca.beq.util.win32.registry.RegistryValue
 * @see java.util.Iterator
 *
 * @author BEQ Technologies Inc.
 * @version $Revision: 1.11 $
 * <pre>
 * $Id: ValueIterator.java,v 1.11 2006/07/10 02:47:46 bayblade Exp $
 * 
 * $Log: ValueIterator.java,v $
 * Revision 1.11  2006/07/10 02:47:46  bayblade
 * Javadoc tweaks
 *
 * Revision 1.10  2006/07/10 02:07:50  bayblade
 * initializeFields has no need to be publicly accessable, and only confuses the API by being public.
 *
 * Revision 1.9  2006/07/06 04:17:12  bayblade
 * Adding dispose() (nee: doCloseRegKey() ) method in an effort to make sure key is closed--but only once, courtesy Dave Kipfer <kipfer@cae.com>
 *
 * Revision 1.8  2005/12/05 03:22:59  bayblade
 * Hacked to accomodate the versioning sync between the jni and the java portions of the code.  Note that a version mismatch on implicit calls will spew data on stderr, and a mismatch on an explicit call will throw a Registry exception, although initialization will have occured anyhow.  As this is error, not common behavior, its not expected to break anything that's not broken anyhow.
 *
 * Revision 1.7  2004/12/20 21:21:30  bayblade
 * Revampled initialization process so 1.0 upgraders weren't so confused;  fixed bug with IBM machines; and accidentally reverted to ANSI formatting instead of K&R in a few spots
 *
 * Revision 1.6  2004/04/20 00:16:18  bayblade
 * Added &lt;pre&gt; tags in headers so change history log 
 * can be sensibly read from javadoc
 *
 * Revision 1.5  2004/04/13 19:17:33  mollinie
 * Call to constructors of the class will fail unless initialization succeeded (in order
 * to enforce the initialize-before-use policy).
 *
 * Revision 1.4  2004/04/02 19:12:52  mollinie
 * Iterator initialization used to occur lazily at the "first" call of any of its implemented methods (hasNext or getNext).
 * However, this approach complicates the code, poses semantic issues with the iterator methods (exceptions could occur while these methods are only allowed to throw NoSuchElementException), and brings nothing.
 * Now, the iterator is initialized natively at instanciation time (method initializeFields) and its hasNext method is not native any longer.
 * Also, the code was reorganized according to structured programing principles, with only one exit point.
 *
 * Revision 1.3  2004/03/29 19:47:42  mollinie
 * Renamed a field previously renamed but accessed from native code under its original name.
 *
 * Revision 1.2  2004/03/27 01:01:01  mollinie
 * Preliminary reformating.
 *
 * Revision 1.1.1.1  2004/03/25 22:05:38  bayblade
 * Initial Import
 * </pre>
 */

public class ValueIterator implements Iterator {


    // implicit initialization. see method for details.
    static {
        try {
            RegistryKey.initialize();
        } catch ( RegistryException re ) {
            re.printStackTrace();
        }
    }

    RegistryKey m_key;

    int m_index;

    int m_hkey;

    int m_maxsize;

    int m_count;


    /**
     * Constructs a new <code>ValueIterator</code> to enumerate values from the
     * specified <code>RegistryKey</code>.
     *
     * @param key The registry key from which to enumerate values.
     */
    public ValueIterator( RegistryKey key ) {
        m_key = key;
        initializeFields();
    }


    private final native void initializeFields();


    /**
     * Returns <code>true</code> if the iteration contains more RegistryValue
     * elements. (In other words, returns <code>true</code> if
     * <code>next</code> would return a RegistryValue element rather than
     * throwing an exception.)
     *
     * @return <code>true</code> if the iterator has more RegistryValue elements.
     */
    public boolean hasNext() {
        return m_index < m_count;
    }


    /**
     * Returns the next RegistryValue element in the iteration.
     *
     * @return the next RegistryValue element in the iteration.
     *
     * @throws NoSuchElementException - if the iteration contains no more
     *         RegistryValue elements.
     */
    public Object next() {
        return m_key.getValue( getNext() );
    }


    /**
     * Returns the next RegistryValue element in the iteration.
     *
     * @return the name of the next RegistryValue element in the iteration.
     *
     * @throws NoSuchElementException - if the iteration contains no more
     *         RegistryValue elements.
     */
    native String getNext();


    /**
     * The optional <code>remove</code> operation is not supported by this
     * Iterator.
     *
     * @throws UnsupportedOperationException - if the remove operation is not
     *         supported by this Iterator.
     */
    public void remove() {
        throw new UnsupportedOperationException(
                "The remove operation is not supported by this Iterator" );
    }
    
    /**
     * This method cleans up underlying native registry.
     * <br>This method should be called when your JVM is not likely to call the 
     * {@link #finalize()} method of this object. 
     * <p><strong>While it is safe to call this method after this Iterator is no 
     * longer in use, the behavior of this Iterator is undefined after this 
     * method is called, but we're pretty certain it won't be pretty.</strong>
     */
    protected native void dispose();


    /**
     * Called by the garbage collector on an object at the end of its life.
     * @see Object#finalize()
     */
    protected void finalize() throws Throwable {
        dispose();
        super.finalize();
    }
    
}