/* ValueIterator.cpp
 *
 * jRegistryKey - A JNI wrapper of the Windows Registry functions.
 * Copyright (c) 2001, BEQ Technologies Inc.
 * 9407 - 27 Avenue
 * Edmonton, Alberta
 * T6N - 1L9 Canada
 * (780) 430-0056
 * (780) 437-6121 (fax)
 * http://www.beq.ca
 *
 * Original Author: Joe Robinson <joe@beq.ca>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author BEQ Technologies Inc.
 * @version $Revision: 1.10 $
 * 
 * $Id: ValueIterator.cpp,v 1.10 2007/03/04 02:30:19 bayblade Exp $
 * 
 * $Log: ValueIterator.cpp,v $
 * Revision 1.10  2007/03/04 02:30:19  bayblade
 * New K&R indentation.
 *
 * Revision 1.9  2006/07/06 05:22:25  bayblade
 * Introducing Win32Exception ...
 * Subclassed RegistryException with new Win32Exception that contains the windows error message, and a method to see the error code as an int rather than a formatted string containing the error code.  The exception message is different, which may break logic that parses this message, but explicitly catching the subclass now allows for greater refinement when processing what actually broke, and is a better solution overall.
 *
 * Revision 1.8  2006/07/06 04:13:12  bayblade
 * added native method synchronization on dispose
 *
 * Revision 1.7  2006/07/06 03:38:01  bayblade
 * Adding dispose() (nee: doCloseRegKey() ) method in an effort to make sure key is closed--but only once, courtesy Dave Kipfer <kipfer@cae.com>
 *
 * Revision 1.6  2005/02/25 22:09:11  bayblade
 * Fix to bug report 1151552.  We now allow "" (and null) for keynames (default name for keys).  While not rigorously tested for jni problems, it does seem to work for simple cases.
 *
 * Revision 1.5  2004/04/06 18:18:23  mollinie
 * Added a parameter to custom function openKey of type REGSAM to indicate what rights to open a key with.
 *
 * Revision 1.4  2004/04/02 23:54:46  mollinie
 * Extracted the common code to extract rootKey and key path and open key into a common method openKey.
 * Moved utility functions prototypes to header jRegistryKey.h
 *
 * Revision 1.3  2004/04/02 20:52:28  mollinie
 * Changed the return type of getRootKey from int to HKEY and reflected the change in the code.
 * Added missing sanity checks in KeyIterator.initializeFields and ValueIterator.initializeFields.
 *
 * Revision 1.2  2004/04/02 19:12:51  mollinie
 * Iterator initialization used to occur lazily at the "first" call of any of its implemented methods (hasNext or getNext).
 * However, this approach complicates the code, poses semantic issues with the iterator methods (exceptions could occur while these methods are only allowed to throw NoSuchElementException), and brings nothing.
 * Now, the iterator is initialized natively at instanciation time (method initializeFields) and its hasNext method is not native any longer.
 * Also, the code was reorganized according to structured programing principles, with only one exit point.
 *
 * Revision 1.1  2004/03/31 23:16:22  mollinie
 * Re-organized the native code organization and build process.
 *
 * Revision 1.2  2004/03/29 19:48:54  mollinie
 * Preliminary reformating.
 *
 * Revision 1.1.1.1  2004/03/25 22:05:38  bayblade
 * Initial Import
 *
 */

#include <windows.h>

#include "jniClassDescriptor.h"
#include "jniFieldDescriptor.h"
#include "jRegistryKey.h"
#include "ca_beq_util_win32_registry_ValueIterator.h"

JNIEXPORT void JNICALL Java_ca_beq_util_win32_registry_ValueIterator_dispose(JNIEnv *env, jobject obj) {
    env->MonitorEnter(obj);
    jfieldID fid_hkey = env->GetFieldID(env->GetObjectClass(obj), "m_hkey", fieldDescriptor::jfd_int);
    HKEY hkey = (HKEY)env->GetIntField(obj, fid_hkey);
    if (hkey != NULL)
        RegCloseKey(hkey);
    env->SetIntField(obj, fid_hkey, (jint) NULL);
    env->MonitorExit(obj);
}

/**
 * startValueIteration()
 *    Helper method to set up RegistryValue enumeration.
 */
JNIEXPORT void JNICALL Java_ca_beq_util_win32_registry_ValueIterator_initializeFields(JNIEnv *env, jobject obj) {
    jclass cls_ValueIterator = env->GetObjectClass(obj);
    jfieldID fid_key = env->GetFieldID(cls_ValueIterator, "m_key", registryDescriptor::jfd_RegistryKey);
    if (fid_key != NULL) {
        jobject obj_RegistryKey = env->GetObjectField(obj, fid_key);
        HKEY hkey = openKey(env, obj_RegistryKey, KEY_READ);
        if (hkey != NULL) {
            DWORD count = 0;
            DWORD maxsize = 0;
            LONG status = RegQueryInfoKey(hkey, NULL, NULL, NULL, NULL, NULL, NULL, &count, &maxsize, NULL, NULL, NULL);
            if(status == ERROR_SUCCESS) {
                jfieldID fid_hkey = env->GetFieldID(cls_ValueIterator, "m_hkey", fieldDescriptor::jfd_int);
                jfieldID fid_maxsize = env->GetFieldID(cls_ValueIterator, "m_maxsize", fieldDescriptor::jfd_int);
                jfieldID fid_index = env->GetFieldID(cls_ValueIterator, "m_index", fieldDescriptor::jfd_int);
                jfieldID fid_count = env->GetFieldID(cls_ValueIterator, "m_count", fieldDescriptor::jfd_int);

                if ((fid_hkey != NULL) && (fid_maxsize != NULL) && (fid_index != NULL) && (fid_count != NULL)) {
                    env->SetIntField(obj, fid_hkey, (jint)hkey);
                    env->SetIntField(obj, fid_maxsize, maxsize + 1);
                    env->SetIntField(obj, fid_index, 0);
                    env->SetIntField(obj, fid_count, count);
                }
            } else {
                throwWin32( env, status );
            }
        }
    }
}

/**
 * ValueIterator::getNext()
 *    Retrieves the NAME of the next value from the registry key.
 */
JNIEXPORT jstring JNICALL Java_ca_beq_util_win32_registry_ValueIterator_getNext(JNIEnv *env, jobject obj) {
    jstring nextValueName = NULL;

    jclass cls_ValueIterator = env->GetObjectClass(obj);
    jfieldID fid_index = env->GetFieldID(cls_ValueIterator, "m_index", fieldDescriptor::jfd_int);
    jfieldID fid_count = env->GetFieldID(cls_ValueIterator, "m_count", fieldDescriptor::jfd_int);

    if ((fid_index != NULL) && (fid_count != NULL)) {
        int index = env->GetIntField(obj, fid_index);
        int count = env->GetIntField(obj, fid_count);
        if(index < count) {
            jfieldID fid_maxsize = env->GetFieldID(cls_ValueIterator, "m_maxsize", fieldDescriptor::jfd_int);
            jfieldID fid_hkey = env->GetFieldID(cls_ValueIterator, "m_hkey", fieldDescriptor::jfd_int);

            if ((fid_maxsize != NULL) && (fid_hkey != NULL)) {
                int maxsize = env->GetIntField(obj, fid_maxsize);
                HKEY hkey = (HKEY)env->GetIntField(obj, fid_hkey);

                char *cNextValueName = (char*)malloc(maxsize);
                if (cNextValueName != NULL) {
                    LONG status = RegEnumValue(hkey, index, cNextValueName, (LPDWORD)&maxsize, NULL, NULL, NULL, NULL);
                    if(status == ERROR_SUCCESS) {
                        nextValueName = env->NewStringUTF(cNextValueName);
                        //      if (nextValueName != NULL) {
                        index++;
                        env->SetIntField(obj, fid_index, index);

                        if(index == count) {
                            RegCloseKey(hkey);
                            env->SetIntField(obj, fid_hkey, (jint) NULL);
                        }
                        //      }
                    } else {
                        if (status != ERROR_MORE_DATA && status != ERROR_NO_MORE_ITEMS) {
                            throwWin32( env, status );
                        }

                        env->SetIntField(obj, fid_index, count);
                        RegCloseKey(hkey);
                        env->SetIntField(obj, fid_hkey, (jint) NULL);
                    }
                    free(cNextValueName);
                } else {
                    jclass cls_OutOfMemoryError = env->FindClass(classDescriptor::jcd_OutOfMemoryError);
                    env->ThrowNew(cls_OutOfMemoryError, "Could not allocate memory.");
                }
            }
        } else {
            jclass cls_NoSuchElementException = env->FindClass(classDescriptor::jcd_utilNoSuchElementException);
            env->ThrowNew(cls_NoSuchElementException, "");
        }
    }
    return nextValueName;
}

